# Decorators are a very powerful and useful tool in Python since it allows programmers to modify the behaviour of a function or class. Decorators allow us to wrap another function in order to extend the behaviour of the wrapped function, without permanently modifying it. 


import time
from random import randint
import os

def log(func):
	def print_func(*args, **kwargs):
		f = open("machine.log", 'a')
		spl = func.__name__.split('_')
		d = ""
		for w in spl:
			d += w[0].upper() + w[1:] + " "
		ts = time.time()
		val = func(*args, **kwargs)
		te = time.time()
		dif = (te - ts)
		if dif > 1:
			t = "{:.3f}s".format(dif)
		else:
			t = "{:.3f}ms".format(dif * 1000)
		f.write('(esommier)Running: ' + d.ljust(15, ' ') + ' [ exec-time = ' + t + ' ]\n')
		f.close
		return val
	return print_func

class CoffeeMachine():
	water_level = 100

	@log
	def start_machine(self):
		if self.water_level > 20:
			return True
		else:
			print("Please add water!")
			return False

	@log
	def boil_water(self):
		return "boiling..."

	@log
	def make_coffee(self):
		if self.start_machine():
			for _ in range(20):
				time.sleep(0.1)
				self.water_level -= 1
			print(self.boil_water())
			print("Coffee is ready!")

	@log
	def add_water(self, water_level):
		time.sleep(randint(1, 5))
		self.water_level += water_level
		print("Blub blub blub...")

if __name__ == "__main__":
	f = open('machine.log', 'w')
	f.close()
	machine = CoffeeMachine()
	for i in range(0, 5):
		machine.make_coffee()

	machine.make_coffee()
	machine.add_water(70)



# This file is essential if you intend to build the actual module
# It contains information about your package, specifically the name of the package, its version, platform-dependencies and a whole lot more.

from setuptools import setup, find_packages
import setuptools

with open("README.md", "r") as f:
	description = f.read()

setuptools.setup(
	name="my_minipack",
	version="1.0.1",
	author="esommier",
	author_email="esommier@student.42.fr",
	# packages=["my_minipack"],
	description="How to create a package in python.",
	long_description=description,
	long_description_content_type="text/markdown",
	# url="[PATH TO BOOTCAMP PYTHON]/module02/tmp_env/lib/python3.7/site-packages",
	license='GPLv3',
	package_dir={"": "src"},
	packages=setuptools.find_packages(where="src"),
	python_requires='>=3.8',
	install_requires=[],
	classifiers=[
		"Development Status :: 3 - Alpha",
		"Intended Audience :: Developers",
		# "Intended Audience :: Students",
		"Topic :: Education",
		# "Topic :: HowTo",
		# "Topic :: Package",
		"License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
		"Programming Language :: Python :: 3",
		"Programming Language :: Python :: 3 :: Only",
	],
)


from time import sleep
from time import time

def ft_progress(listy):
	start1 = time()
	for n in listy:
		start2 = time()
		yield n
		diff1 = (time() - start2) * len(listy)
		diff2 = time() - start1
		l = 20
		d = "=" * round(n * l / len(listy)) + ">" + " " * round(l - (n * l / len(listy)))
		print("ETA: {:.2f}s [ {:.2f}%][{}] {}/{} | elapsed time {:.2f}s".format(diff1, n / (len(listy) / 100), d, n + 1, len(listy), diff2), end='\r')

